##
# An object for storing information about a field filter.
# <BR/>Permitted values for operator:
# <UL>
# <LI>'eq' (equals)</LI>
# <LI>'lt' (less than)</LI>
# <LI>'gt' (greater than)</LI>
# <LI>'not' (not)</LI>
# <LI>'le' (less than or equal to)</LI>
# <LI>'ge' (greater than or equal to)</LI>
# <LI>'in' (match against a comma-separated list of values)</LI>
# <LI>'null' (value is null)</LI>
# <LI>'notnull' (value is not null)</LI>
# <LI>'contains'</LI>
# <LI>'startswith'</LI>
# <LI>'endswith'</LI>
# <LI>'like' (partial match) - for example, field like 'hello%'</LI>
# <LI>'sl' (sounds like) - implements a SOUNDEX() query</LI>
# <LI>'co' (char order) - for example, field co 'hello' "field like '%h%e%l%l%o%'"</LI>
# <LI>'com' (compound - see MMSearch#compound_order)</LI>
# </UL>
# <BR/>Default: 'eq'
# <BR/>Permitted values for options:
# <UL>
# <LI>wildcard_punctuation ("john's cars" will search for "john%s cars")</LI>
# <LI>remove_punctuation ("john's cars" will search for "johns cars")</LI>
# <LI>wildcard_space ("john's cars" will search for "john's%cars")</LI>
# <LI>remove_space ("john's cars" will search for "john'cars")</LI>
# </UL>
# <BR/>Default: none
# <BR/>NOTE: If wildcard_punctuation and remove_punctuation, or wildcard_space and remove_space are used together, the wildcard_* option will take precedence
#
# @param {String} field The name of the field to query.
# @param {String} operator The operator to be used.
# @param {Object} value The value to be used.
# @param {String} options Array of options that can be used with eq, contains, startswith, endswith, like, sl, co or com operators.  Can also be a comma-separated list.
class MMSearchFilter
  attr_accessor :field, :operator, :value, :options

  def initialize( field, operator, value = nil, options = nil )
    # The name of the field to query.
    # @type String
    self.field = field

    # The operator to be used.
    # @type String
    self.operator = operator

    # The value to be used.
    # @type Object
    self.value = value

    # Array of options that can be used with eq, contains, startswith, endswith, like, sl, co or com operators.  Can also be a comma-separated list.
    # @type String
    self.options = options

end

# Function to return a simple string representation of the object.
# @return Formatted version of the object as "[field:operator:value:<comma-separated list of options>]".
# @type String
  def to_s
    if self.options.is_a?( Array )
      options_string = self.options.join(',')
    else
      options_string = self.options
    end
    return '['+self.field+':'+self.operator+':'+self.value+':'+options_string+']'
  end

# Function to return a copy of this object in another object.
# @return A new object containing the same data as this object.
# @type MMSearchFilter
  def copy
    return MMSearchFilter.new( self.field, self.operator, self.value, self.options )
  end

# Function to return a string representation of the object to be used within
# an MM-API query call.
# @return Query string formatted version of the object 
# @type String
  def toAPIQueryString( postfix_number = "" )
    url = ''
    postfix = ''
    if postfix_number and postfix_number != ""
      postfix = '_' + postfix_number
    end
    url += '&fieldname' + postfix + '=' + URI.escape(self.field,/[^0-9a-zA-Z\-]/)
    if self.operator
      url += '&operator' + postfix + '=' + URI.escape(self.operator,/[^0-9a-zA-Z\-]/)
    end
    url += '&value' + postfix + '=' + URI.escape(self.value,/[^0-9a-zA-Z\-]/)
    
    options_string = ''
    if self.options
      options_string = '&fieldFilterOptions' + postfix + '='
      if self.options.is_a?( Array )
        options_string += URI.escape(self.options.join(','),/[^0-9a-zA-Z\-]/)
      else
        options_string += URI.escape(self.options,/[^0-9a-zA-Z\-]/)
      end
    end
    
    url += options_string
    
    return url
  end
end
